/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2016 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.ui.internal.introspection.details.handler;

import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.ScrolledComposite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.swt.widgets.TreeColumn;
import org.eclipse.ui.dialogs.PatternFilter;
import org.fortiss.tooling.kernel.introspection.items.ConnectionCompositorServiceIntrospectionDetailsItem;
import org.fortiss.tooling.kernel.introspection.items.EObjectAware2IntrospectionDetailsItemBase;
import org.fortiss.tooling.kernel.service.IConnectionCompositorService;

/**
 * Introspection UI handler for the {@link IConnectionCompositorService}.
 * 
 * @author hoelzl
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating RED Hash:
 */
public final class ConnectionCompositorServiceIntrospectionDetailsUIHandler extends
		EObjectAware2IntrospectionDetailsUIHandlerBase {

	/** {@inheritDoc} */
	@Override
	public Control createComposite(ScrolledComposite parent) {
		String heading = "Type to serach registered connection compositors:";
		String footer =
				"Number of currently registered connection compositors: " +
						getInputObject().countHandlers();
		return createFilteredTreeInTabFolder(parent, heading, footer, "Registered Compositors");
	}

	/** {@inheritDoc} */
	@Override
	protected void createTreeColumns(Tree tree) {
		TreeColumn col0 = new TreeColumn(tree, SWT.LEFT);
		col0.setText("Source / Target / Compositor");
		col0.setWidth(250);

		TreeColumn col1 = new TreeColumn(tree, SWT.LEFT);
		col1.setText("Compositor Class");
		col1.setWidth(400);

		TreeColumn col2 = new TreeColumn(tree, SWT.LEFT);
		col2.setText("Source EObject Class");
		col2.setWidth(400);

		TreeColumn col3 = new TreeColumn(tree, SWT.LEFT);
		col3.setText("Target EObject Class");
		col3.setWidth(400);
	}

	/** {@inheritDoc} */
	@Override
	protected ITreeContentProvider createContentProvider() {
		return new EObjectAware2TreeContentProviderBase() {
			@Override
			protected EObjectAware2IntrospectionDetailsItemBase<?> getInputObject() {
				return ConnectionCompositorServiceIntrospectionDetailsUIHandler.this
						.getInputObject();
			}
		};
	}

	/** {@inheritDoc} */
	@Override
	protected ITableLabelProvider createLabelProvider() {
		return new EObjectAware2TableLabelProvider();
	}

	/** {@inheritDoc} */
	@Override
	protected PatternFilter createPatternFilter() {
		PatternFilter pf = new TableViewerPatternFilter();
		pf.setIncludeLeadingWildcard(true);
		return pf;
	}

	/** {@inheritDoc} */
	@Override
	protected ConnectionCompositorServiceIntrospectionDetailsItem getInputObject() {
		return (ConnectionCompositorServiceIntrospectionDetailsItem)dataItem;
	}
}
